@@ -17,14 +17,14 @@
 #include "sd-journal.h"
 
 #include "alloc-util.h"
-//#include "fd-util.h"
+#include "errno-util.h"
+#include "fd-util.h"
 #include "io-util.h"
-//#include "memfd-util.h"
-//#include "socket-util.h"
+#include "memfd-util.h"
+#include "socket-util.h"
 #include "stdio-util.h"
 #include "string-util.h"
-//#include "tmpfile-util.h"
-#include "util.h"
+#include "tmpfile-util.h"
 /// Additional includes needed by elogind
 #include <syslog.h>
 
