@@ -18,6 +18,7 @@
 #include "hexdecoct.h"
 #include "io-util.h"
 #include "macro.h"
+#include "memory-util.h"
 #include "missing.h"
 #include "path-util.h"
 #include "process-util.h"
@@ -28,7 +29,6 @@
 #include "string-util.h"
 #include "user-util.h"
 #include "utf8.h"
-#include "util.h"
 
 #define SNDBUF_SIZE (8*1024*1024)
 
@@ -160,7 +160,6 @@ static int bus_socket_write_auth(sd_bus *b) {
 static int bus_socket_auth_verify_client(sd_bus *b) {
         char *d, *e, *f, *start;
         sd_id128_t peer;
-        unsigned i;
         int r;
 
         assert(b);
@@ -205,9 +204,8 @@ static int bus_socket_auth_verify_client(sd_bus *b) {
                  */
                 if (memcmp(b->rbuffer, "OK ", 3))
                         return -EPERM;
-        } else {
+        } else
                 return -EPERM;
-        }
 
         /* Now check the OK line. */
 
@@ -219,7 +217,7 @@ static int bus_socket_auth_verify_client(sd_bus *b) {
 
         b->auth = b->anonymous_auth ? BUS_AUTH_ANONYMOUS : BUS_AUTH_EXTERNAL;
 
-        for (i = 0; i < 32; i += 2) {
+        for (unsigned i = 0; i < 32; i += 2) {
                 int x, y;
 
                 x = unhexchar(d[2 + 3 + i]);
@@ -1147,8 +1145,11 @@ static int bus_socket_make_message(sd_bus *bus, size_t size) {
         bus->fds = NULL;
         bus->n_fds = 0;
 
-        if (t)
-                bus->rqueue[bus->rqueue_size++] = t;
+        if (t) {
+                t->read_counter = ++bus->read_counter;
+                bus->rqueue[bus->rqueue_size++] = bus_message_ref_queued(t, bus);
+                sd_bus_message_unref(t);
+        }
 
         return 1;
 }
