@@ -344,7 +344,7 @@ static int file_of_seat(const char *seat, char **_p) {
                 if (!filename_is_valid(seat))
                         return -EINVAL;
 
-                p = strappend("/run/systemd/seats/", seat);
+                p = path_join("/run/systemd/seats", seat);
         } else {
                 _cleanup_free_ char *buf = NULL;
 
@@ -352,9 +352,8 @@ static int file_of_seat(const char *seat, char **_p) {
                 if (r < 0)
                         return r;
 
-                p = strappend("/run/systemd/seats/", buf);
+                p = path_join("/run/systemd/seats", buf);
         }
-
         if (!p)
                 return -ENOMEM;
 
@@ -457,7 +456,7 @@ static int file_of_session(const char *session, char **_p) {
                 if (!session_id_valid(session))
                         return -EINVAL;
 
-                p = strappend("/run/systemd/sessions/", session);
+                p = path_join("/run/systemd/sessions", session);
         } else {
                 _cleanup_free_ char *buf = NULL;
 
@@ -465,7 +464,7 @@ static int file_of_session(const char *session, char **_p) {
                 if (r < 0)
                         return r;
 
-                p = strappend("/run/systemd/sessions/", buf);
+                p = path_join("/run/systemd/sessions", buf);
         }
 
         if (!p)
@@ -848,7 +847,7 @@ _public_ int sd_get_uids(uid_t **users) {
                                 uid_t *t;
 
                                 n = MAX(16, 2*r);
-                                t = realloc(l, sizeof(uid_t) * n);
+                                t = reallocarray(l, sizeof(uid_t), n);
                                 if (!t)
                                         return -ENOMEM;
 
