@@ -19,7 +19,7 @@
 
 
 #include "elogind-dbus.h"
-//#include "exec-util.h"
+#include "exec-util.h"
 #include "process-util.h"
 #include "sd-messages.h"
 #include "sleep.h"
@@ -96,7 +96,7 @@ static int run_helper(Manager* m, const char *helper, const char *arg_verb) {
         m->callback_failed = false;
         m->callback_must_succeed = m->allow_poweroff_interrupts;
 
-        r = execute_directories(dirs, DEFAULT_TIMEOUT_USEC, gather_output, gather_args, verb_args, NULL);
+        r = execute_directories(dirs, DEFAULT_TIMEOUT_USEC, gather_output, gather_args, verb_args, NULL, EXEC_DIR_NONE);
 
         if ( m->callback_must_succeed && ((r < 0) || m->callback_failed) ) {
                 e = asprintf(&l, "A shutdown script in %s failed! [%d]\n"
@@ -204,6 +204,9 @@ int execute_shutdown_or_sleep(
         /* no more pending actions, whether this failed or not */
         m->pending_action = HANDLE_IGNORE;
 
+        if (r < 0)
+                return r;
+
         /* As elogind can not rely on a systemd manager to call all
          * sleeping processes to wake up, we have to tell them all
          * by ourselves. */
@@ -214,10 +217,9 @@ int execute_shutdown_or_sleep(
                 m->action_what = w;
 
         /* Make sure the lid switch is ignored for a while */
-        if (0 == r)
-                manager_set_lid_switch_ignore(m, now(CLOCK_MONOTONIC) + m->holdoff_timeout_usec);
+        manager_set_lid_switch_ignore(m, now(CLOCK_MONOTONIC) + m->holdoff_timeout_usec);
 
-        return r;
+        return 0;
 }
 
 int manager_scheduled_shutdown_handler(
