@@ -3,17 +3,18 @@
 #include <stdint.h>
 #include <sys/mount.h>
 
-//#include "sd-bus.h"
+#include "sd-bus.h"
 
-//#include "bus-error.h"
+#include "bus-error.h"
 #include "fs-util.h"
+#include "format-util.h"
 #include "label.h"
-//#include "main-func.h"
+#include "main-func.h"
 #include "mkdir.h"
 #include "mountpoint-util.h"
 #include "path-util.h"
 #include "rm-rf.h"
-//#include "selinux-util.h"
+#include "selinux-util.h"
 #include "smack-util.h"
 #include "stdio-util.h"
 #include "string-util.h"
@@ -200,7 +201,7 @@ static int run(int argc, char *argv[]) {
                 return log_error_errno(r, "Could not initialize labelling: %m\n");
 #else
 int user_runtime_dir(const char *verb, User *u) {
-        int r;
+        int r = 0;
 
         assert_se(verb);
         assert_se(u);
