@@ -5,30 +5,38 @@
 ***/
 
 #include <errno.h>
+#include <fcntl.h>
 #include <getopt.h>
 #include <linux/fiemap.h>
+#include <poll.h>
 #include <stdio.h>
+#include <sys/stat.h>
+#include <sys/types.h>
+#include <sys/timerfd.h>
+#include <unistd.h>
 
 #include "sd-messages.h"
 
+//#include "btrfs-util.h"
 #include "def.h"
-//#include "exec-util.h"
+#include "exec-util.h"
 #include "fd-util.h"
+#include "format-util.h"
 #include "fileio.h"
-//#include "log.h"
-//#include "main-func.h"
+#include "log.h"
+#include "main-func.h"
 #include "parse-util.h"
-//#include "pretty-print.h"
+#include "pretty-print.h"
 #include "sleep-config.h"
 #include "stdio-util.h"
 #include "string-util.h"
 #include "strv.h"
+#include "time-util.h"
 #include "util.h"
 
 /// Additional includes needed by elogind
 #include "exec-elogind.h"
 #include "sleep.h"
-#include "static-destruct.h"
 #include "utmp-wtmp.h"
 
 static char* arg_verb = NULL;
@@ -94,6 +102,16 @@ static int write_hibernate_location_info(void) {
         if (r < 0)
                 return log_debug_errno(errno, "Unable to stat %s: %m", device);
 
+#if 0 /// UNNEEDED by elogind
+        r = btrfs_is_filesystem(fd);
+        if (r < 0)
+                return log_error_errno(r, "Error checking %s for Btrfs filesystem: %m", device);
+#endif // 0
+
+        if (r)
+                return log_debug_errno(SYNTHETIC_ERRNO(EOPNOTSUPP),
+                                       "Unable to calculate swapfile offset when using Btrfs: %s", device);
+
         r = read_fiemap(fd, &fiemap);
         if (r < 0)
                 return log_debug_errno(r, "Unable to read extent map for '%s': %m", device);
@@ -107,11 +125,15 @@ static int write_hibernate_location_info(void) {
         if (r < 0)
                 return log_debug_errno(r, "Failed to write offset '%s': %m", offset_str);
 
+        log_debug("Wrote calculated resume_offset value to /sys/power/resume_offset: %s", offset_str);
+
         xsprintf(device_str, "%lx", (unsigned long)stb.st_dev);
         r = write_string_file("/sys/power/resume", device_str, WRITE_STRING_FILE_DISABLE_BUFFER);
         if (r < 0)
                 return log_debug_errno(r, "Failed to write device '%s': %m", device_str);
 
+        log_debug("Wrote device id to /sys/power/resume: %s", device_str);
+
         return 0;
 }
 
@@ -157,6 +179,32 @@ static int write_state(FILE **f, char **states) {
         return r;
 }
 
+static int configure_hibernation(void) {
+        _cleanup_free_ char *resume = NULL, *resume_offset = NULL;
+        int r;
+
+        /* check for proper hibernation configuration */
+        r = read_one_line_file("/sys/power/resume", &resume);
+        if (r < 0)
+                return log_debug_errno(r, "Error reading from /sys/power/resume: %m");
+
+        r = read_one_line_file("/sys/power/resume_offset", &resume_offset);
+        if (r < 0)
+                return log_debug_errno(r, "Error reading from /sys/power/resume_offset: %m");
+
+        if (!streq(resume_offset, "0") && !streq(resume, "0:0")) {
+                log_debug("Hibernating using device id and offset read from /sys/power/resume: %s and /sys/power/resume_offset: %s", resume, resume_offset);
+                return 0;
+        } else if (!streq(resume, "0:0")) {
+                log_debug("Hibernating using device id read from /sys/power/resume: %s", resume);
+                return 0;
+        } else if (!streq(resume_offset, "0"))
+                log_debug("Found offset in /sys/power/resume_offset: %s; no device id found in /sys/power/resume; ignoring offset", resume_offset);
+
+        /* if hibernation is not properly configured, attempt to calculate and write values */
+        return write_hibernate_location_info();
+}
+
 #if 0 /// elogind uses the values stored in its manager instance
 static int execute(char **modes, char **states) {
 #else
@@ -181,6 +229,7 @@ static int execute(Manager *m, const char *verb) {
                         streq(arg_verb, "hibernate") ? m->hibernate_state   :
                                                        m->hybrid_sleep_state;
 #endif // 0
+
         char *arguments[] = {
                 NULL,
                 (char*) "pre",
@@ -205,21 +254,22 @@ static int execute(Manager *m, const char *verb) {
 
         /* Configure the hibernation mode */
         if (!strv_isempty(modes)) {
-                r = write_hibernate_location_info();
+                r = configure_hibernation();
                 if (r < 0)
-                        return log_error_errno(r, "Failed to write hibernation disk offset: %m");
+                        return log_error_errno(r, "Failed to prepare for hibernation: %m");
+
                 r = write_mode(modes);
                 if (r < 0)
                         return log_error_errno(r, "Failed to write mode to /sys/power/disk: %m");;
         }
 
 #if 0 /// elogind needs its own callbacks to enable cancellation by erroneous scripts
-        execute_directories(dirs, DEFAULT_TIMEOUT_USEC, NULL, NULL, arguments, NULL);
+        (void) execute_directories(dirs, DEFAULT_TIMEOUT_USEC, NULL, NULL, arguments, NULL, EXEC_DIR_PARALLEL | EXEC_DIR_IGNORE_ERRORS);
 #else
         m->callback_failed = false;
         m->callback_must_succeed = m->allow_suspend_interrupts;
 
-        r = execute_directories(dirs, DEFAULT_TIMEOUT_USEC, gather_output, gather_args, arguments, NULL);
+        r = execute_directories(dirs, DEFAULT_TIMEOUT_USEC, gather_output, gather_args, arguments, NULL, EXEC_DIR_NONE);
 
         if ( m->callback_must_succeed && ((r < 0) || m->callback_failed) ) {
                 e = asprintf(&l, "A sleep script in %s failed! [%d]\n"
@@ -261,106 +311,86 @@ static int execute(Manager *m, const char *verb) {
                            "SLEEP=%s", arg_verb);
 
         arguments[1] = (char*) "post";
-        execute_directories(dirs, DEFAULT_TIMEOUT_USEC, NULL, NULL, arguments, NULL);
+        (void) execute_directories(dirs, DEFAULT_TIMEOUT_USEC, NULL, NULL, arguments, NULL, EXEC_DIR_PARALLEL | EXEC_DIR_IGNORE_ERRORS);
 
         return r;
 }
 
-static int rtc_read_time(uint64_t *ret_sec) {
-        _cleanup_free_ char *t = NULL;
-        int r;
-
-        r = read_one_line_file("/sys/class/rtc/rtc0/since_epoch", &t);
-        if (r < 0)
-                return log_error_errno(r, "Failed to read RTC time: %m");
-
-        r = safe_atou64(t, ret_sec);
-        if (r < 0)
-                return log_error_errno(r, "Failed to parse RTC time '%s': %m", t);
-
-        return 0;
-}
-
-static int rtc_write_wake_alarm(uint64_t sec) {
-        char buf[DECIMAL_STR_MAX(uint64_t)];
-        int r;
-
-        xsprintf(buf, "%" PRIu64, sec);
-
-        r = write_string_file("/sys/class/rtc/rtc0/wakealarm", buf, WRITE_STRING_FILE_DISABLE_BUFFER);
-        if (r < 0)
-                return log_error_errno(r, "Failed to write '%s' to /sys/class/rtc/rtc0/wakealarm: %m", buf);
-
-        return 0;
-}
 
 #if 0 /// elogind uses the values stored in its manager instance
-static int execute_s2h(usec_t hibernate_delay_sec) {
-
-        _cleanup_strv_free_ char **hibernate_modes = NULL, **hibernate_states = NULL,
-                                 **suspend_modes = NULL, **suspend_states = NULL;
+static int execute_s2h(const SleepConfig *sleep_config) {
 #else
 static int execute_s2h(Manager *m) {
         assert(m);
 
         usec_t hibernate_delay_sec = m->hibernate_delay_sec;
 #endif // 0
-        usec_t original_time, wake_time, cmp_time;
+        _cleanup_close_ int tfd = -1;
+        char buf[FORMAT_TIMESPAN_MAX];
+        struct itimerspec ts = {};
+        struct pollfd fds;
         int r;
 
 #if 0 /// Already parsed by elogind config
-        r = parse_sleep_config("suspend", NULL, &suspend_modes, &suspend_states, NULL);
-        if (r < 0)
-                return r;
-
-        r = parse_sleep_config("hibernate", NULL, &hibernate_modes, &hibernate_states, NULL);
-        if (r < 0)
-                return r;
+        assert(sleep_config);
 #endif // 0
 
-        r = rtc_read_time(&original_time);
-        if (r < 0)
-                return r;
+        tfd = timerfd_create(CLOCK_BOOTTIME_ALARM, TFD_NONBLOCK|TFD_CLOEXEC);
+        if (tfd < 0)
+                return log_error_errno(errno, "Error creating timerfd: %m");
 
-        wake_time = original_time + DIV_ROUND_UP(hibernate_delay_sec, USEC_PER_SEC);
-        r = rtc_write_wake_alarm(wake_time);
-        if (r < 0)
-                return r;
+#if 0 /// elogind uses the values from its manager
+        log_debug("Set timerfd wake alarm for %s",
+                  format_timespan(buf, sizeof(buf), sleep_config->hibernate_delay_sec, USEC_PER_SEC));
 
-        log_debug("Set RTC wake alarm for %" PRIu64, wake_time);
+        timespec_store(&ts.it_value, sleep_config->hibernate_delay_sec);
+#else
+        log_debug("Set timerfd wake alarm for %s",
+                  format_timespan(buf, sizeof(buf), hibernate_delay_sec, USEC_PER_SEC));
+
+        timespec_store(&ts.it_value, hibernate_delay_sec);
+#endif // 0
+
+        r = timerfd_settime(tfd, 0, &ts, NULL);
+        if (r < 0)
+                return log_error_errno(errno, "Error setting hibernate timer: %m");
 
 #if 0 /// elogind uses its manager instance values
-        r = execute(suspend_modes, suspend_states);
+        r = execute(sleep_config->suspend_modes, sleep_config->suspend_states);
 #else
         r = execute(m, "suspend");
 #endif // 0
         if (r < 0)
                 return r;
 
-        /* Reset RTC right-away */
-        r = rtc_write_wake_alarm(0);
+        fds = (struct pollfd) {
+                .fd = tfd,
+                .events = POLLIN,
+        };
+        r = poll(&fds, 1, 0);
         if (r < 0)
-                return r;
+                return log_error_errno(errno, "Error polling timerfd: %m");
 
-        r = rtc_read_time(&cmp_time);
-        if (r < 0)
-                return r;
+        tfd = safe_close(tfd);
 
-        log_debug("Woke up at %"PRIu64, cmp_time);
-
-        if (cmp_time < wake_time) /* We woke up before the alarm time, we are done. */
+        if (!FLAGS_SET(fds.revents, POLLIN)) /* We woke up before the alarm time, we are done. */
                 return 0;
 
         /* If woken up after alarm time, hibernate */
+        log_debug("Attempting to hibernate after waking from %s timer",
 #if 0 /// elogind uses its manager instance values
-        r = execute(hibernate_modes, hibernate_states);
+                  format_timespan(buf, sizeof(buf), sleep_config->hibernate_delay_sec, USEC_PER_SEC));
+
+        r = execute(sleep_config->hibernate_modes, sleep_config->hibernate_states);
 #else
+                  format_timespan(buf, sizeof(buf), hibernate_delay_sec, USEC_PER_SEC));
+
         r = execute(m, "hibernate");
 #endif // 0
         if (r < 0) {
                 log_notice("Couldn't hibernate, will try to suspend again.");
 #if 0 /// elogind uses its manager instance values
-                r = execute(suspend_modes, suspend_states);
+                r = execute(sleep_config->suspend_modes, sleep_config->suspend_states);
 #else
                 r = execute(m, "suspend");
 #endif // 0
@@ -449,8 +479,8 @@ static int parse_argv(int argc, char *argv[]) {
 
 static int run(int argc, char *argv[]) {
         bool allow;
-        _cleanup_strv_free_ char **modes = NULL, **states = NULL;
-        usec_t delay = 0;
+        char **modes = NULL, **states = NULL;
+        _cleanup_(free_sleep_configp) SleepConfig *sleep_config = NULL;
         int r;
 
         log_setup_service();
@@ -459,7 +489,11 @@ static int run(int argc, char *argv[]) {
         if (r <= 0)
                 return r;
 
-        r = parse_sleep_config(arg_verb, &allow, &modes, &states, &delay);
+        r = parse_sleep_config(&sleep_config);
+        if (r < 0)
+                return r;
+
+        r = sleep_settings(arg_verb, sleep_config, &allow, &modes, &states);
         if (r < 0)
                 return r;
 
@@ -469,7 +503,7 @@ static int run(int argc, char *argv[]) {
                                        arg_verb);
 
         if (streq(arg_verb, "suspend-then-hibernate"))
-                return execute_s2h(delay);
+                return execute_s2h(sleep_config);
         else
                 return execute(modes, states);
 }
