@@ -7,6 +7,7 @@
 #include "fd-util.h"
 #include "fileio.h"
 #include "macro.h"
+#include "memory-util.h"
 #include "path-util.h"
 #include "process-util.h"
 #include "random-util.h"
@@ -14,7 +15,6 @@
 #include "string-util.h"
 #include "tests.h"
 #include "tmpfile-util.h"
-#include "util.h"
 
 static void test_close_many(void) {
         int fds[3];
